@@ -38,7 +38,7 @@
  * September, 2001
  *
  * Derived from ratmenu and 9menu.
- * Zrajm C Akfohg <zrajm@klingonska.org>
+ * Zrajm C Akfohg <ratmen-mail@klingonska.org>
  *
  */
 
@@ -144,7 +144,7 @@ void version(void);
 
 
 /*
- * Event loop monitor thingy stolen from: (use option `--debug')
+ * Event loop monitor thingy (use option `--debug') stolen from:
  * http://www-h.eng.cam.ac.uk/help/tpl/graphics/X/X11R5/node41.html
  */
 static char *event_names[] = {
@@ -572,7 +572,6 @@ void items(int start, int count, char **arg) {
 
 int HandleXError(Display *dpy, XErrorEvent *event) {
 /*    gXErrorFlag = 1;*/
-    
     return 0;
 }
 
@@ -666,8 +665,6 @@ int main(int argc, char **argv) {
 
     run_menu(cur_item);
 
-
-
     /* XCloseDisplay() cannot generate BadWindow */
     XCloseDisplay(dpy);
     exit(0);
@@ -708,12 +705,12 @@ void reap(int s) {
 
 
 void version(void) {
-    printf("%s (for Ratpoison) 2.2.2 (compiled 28 March 2003)\n"
+    printf("%s (for Ratpoison) 2.2.3 (compiled 25 October 2007)\n"
         "Written by Arnold Robbins & David Hogan (1994-1995),\n"
         "John M. O'Donnell (1997), Jonathan Walther (2001) and\n"
-        "Zrajm C Akfohg (2003).\n"
+        "Zrajm C Akfohg (2003, 2007).\n"
         "\n"
-        "Copyright 1994-1995, 1997, 2001, 2003 by respective author.\n"
+        "Copyright 1994-1995, 1997, 2001, 2003, 2007 by respective author.\n"
         "Distributed under the GNU Public License.\n", progname);
     exit(0);
 }
@@ -724,30 +721,35 @@ void help(void) {
     printf("Usage: %s [OPTION]... MENUITEM COMMAND ...\n"
         "   or: %s [OPTION]... {-d##|--delimiter ##} MENUITEM##COMMAND ...\n"
         "Create a simple menu in a separate window and run user selected command.\n"
-        "\n"
+        "\n", progname, progname);
+    printf(
         "  -l, -c, -r,                      set window text alignment (Xresource)\n"
         "        --align {left|center|right}\n"
         "  -b, --back PREVMENU              command to run on `back'\n"
         "      --background BGCOLOR         set background color (Xresource)\n"
         "  -C, --class CLASSNAME            CLASSNAME of Xresources to look for\n"
         "      --debug                      enable debug output on stderr\n"
-        "  -d, --delimiter DELIM            pair up each MENUITEM and COMMAND\n"
+        "  -d, --delimiter DELIM            pair up each MENUITEM and COMMAND\n");
+    printf(
         "                                   in one argument separated by DELIM\n"
         "  -D, --display DISPLAYNAME        open menu on named X display\n"
         "      --foreground FGCOLOR         set foreground color (Xresource)\n"
         "  -F, --font FNAME                 Use X font FNAME (Xresource)\n"
         "  -h, --help                       display this help and exit\n"
-        "  i, --item POSITION              set MENUITEM selected on startup\n"
-        "      --mouse / --no-mouse         enable/disable mouse support (Xresource)\n"
+        "  -i, --item POSITION              set MENUITEM selected on startup\n"
+        "      --mouse / --no-mouse         enable/disable mouse support (Xresource)\n");
+    printf(
         "  -p, --print                      print COMMAND on stdout, don't execute\n"
         "  -o, --scroll-offset ITEMS        border distance for scrolling (Xresource)\n"
         "  -S, --shell SHELL                SHELL in which to run commands\n"
         "  -s, --style {snazzy|dreary}      select menu style (Xresource)\n"
         "  -t, --title NAME                 set window title to NAME\n"
         "      --unfocus-exit /             enable/disable exit on loosing window\n"
-        "        --no-unfocus-exit            focus (Xresource)\n"
+        "        --no-unfocus-exit            focus (Xresource)\n");
+    printf(
         "  -V, --version                    output version information and exit\n"
-        "\n", progname, progname);
+        "\n");
+
     exit(0);
 }
 
@@ -796,7 +798,7 @@ void run_menu(int cur) {
      * mouse-related X events to happen, here in this XSelectInput(),
      * rather than modifying the Event test loop below.
      */
-     XSelectInput(dpy, menuwin,
+    XSelectInput(dpy, menuwin,
         (mouse_on == True ? MenuMask : MenuMaskNoMouse));
 
     XMapWindow(dpy, menuwin);
@@ -819,7 +821,7 @@ void run_menu(int cur) {
 
             case LeaveNotify:
                 if (mousing == False) break;
-                fprintf(stderr, "going out\n");
+                /* fprintf(stderr, "going out\n"); */
                 cur = cur_store;
                 redraw_mouse(cur, high, wide);
                 mousing = False;
@@ -1533,11 +1535,11 @@ Hogan and Arnold Robbins. Rich Salz motivated the B<-shell> option. Jonathan
 Walther modified this code to play nicely with the ratpoison window manager by
 removing handling of mouse events and iconification.
 
-Zrajm C Akfogh <zrajm@klingonska.org> changed command line syntax into the more
-standard getopts, added scrolling capacity if menu is to large to fit all at
-once, added B<--delimiter>, B<--item>, B<--print>, B<--scroll-offset> and
-B<--unfocus-exit> options and X resource support (Yay! No need to specify those
-longish font-thingies on the command line any more!).
+Zrajm C Akfogh <ratmen-mail@klingonska.org> changed command line syntax into
+the more standard getopts, added scrolling capacity if menu is to large to fit
+all at once, added B<--delimiter>, B<--item>, B<--print>, B<--scroll-offset>
+and B<--unfocus-exit> options and X resource support (Yay! No need to specify
+those longish font-thingies on the command line any more!).
 
 The name `ratmen' is both an abbreviation of `ratmenu' (from which this program
 is heavily derived) and a reference to the fact that *I* don't have any
@@ -1545,16 +1547,33 @@ religious reasons for not using the rodent (I like the keyboard, but I also
 like freedom of choice).
 
 
+=head1 FUTURE
+
+I have not activelly made any changes to this program for several years, though
+it was originally my intention to write a program that works both under X, and
+in the console.
+
+I later wrote B<termmen>, which much closer resembles my intentions for
+B<ratmen>, but unfortunately only works in the console or terminal (i.e. does
+not pop up a window of its own under X). Both B<termmen> and this program is
+available from L<http://zrajm.klingonska.org/programs/>.
+
+This program is written in C, which is not my native language, while B<Termmen>
+is written as a zsh script. If anyone would like to continue development of
+B<ratmen>, or a have a patch they'd like applied. Please feel free to send it
+to me at ratmen-mail@klingonska.org.
+
+
 =head1 HISTORY
 
-[2003-02-21, 12.47-16.18] Implemented `--item' for choosing initially selected
+[2003-02-21, 12:47-16:18] Implemented `--item' for choosing initially selected
 item. Menu item and command is now given in the same string, separated with :,
 which makes for nicer error detection and lesser errors since it is a bit
 easier to keep track of what does which on the command line.
 
-[2003-02-22, 15.38-16.43]
+[2003-02-22, 15:38-16:43]
 
-[2003-02-23, 02.19-02.36] Implemented `--delimiter' which now must be used to
+[2003-02-23, 02:19-02:36] Implemented `--delimiter' which now must be used to
 get the "menuitem:command" (as opposed to "menuitem" "command") behaviour.
 This, because I realised that when called from a script using an on-the-fly
 generated menu it can be quite tedious to make sure that the delimiter does
@@ -1562,24 +1581,24 @@ not occur in the `menuitem' string (and thus fuck up the menu). Of course in
 handwritten menus the delimiter approach is easier to handle, hence I allow
 for both. Delimiter now also may be more than one character long.
 
-[2003-02-23, 19.08-20.30] Implemented B<--unfocus-exit> which exits the menu if
+[2003-02-23, 19:08-20:30] Implemented B<--unfocus-exit> which exits the menu if
 it's window is unfocused.
 
-[2003-02-23, 21.12-21.22]
+[2003-02-23, 21:12-21:22]
 
-[2003-02-24, 00.22-00.49]
+[2003-02-24, 00:22-00:49]
 
-[2003-02-24, 11.38-17.47] Now reads some options regulating appearance and
+[2003-02-24, 11:38-17:47] Now reads some options regulating appearance and
 behaviour from X resources in addition to the command line.
 
-[2003-02-25, 18.47-22.20] Wrote POD. At last found a really good name for the
+[2003-02-25, 18:47-22:20] Wrote POD. At last found a really good name for the
 product. The B<ratmen>.
 
-[2003-02-26, 01.38-09.15] Now reads command line option using getopt; thusly
+[2003-02-26, 01:38-09:15] Now reads command line option using getopt; thusly
 supports both --long-options and short ones (and bundles of short ones). Yay!
 Still haven't been able to get the B<--unfocus-exit> to work properly.
 
-[2003-02-26, 13.27-14.34] Fixxed B<--unfocus-exit> so that it now really makes
+[2003-02-26, 13:27-14:34] Fixxed B<--unfocus-exit> so that it now really makes
 the menu die of unfocus. Unfortunately, however a `BadWindow' message is
 generated by X when doing so, which looks quite ugly in B<ratpoison> (which
 faithfully reports the error). I simply haven't been able to figure out what
@@ -1587,23 +1606,23 @@ command is causing this, but my best guess is that there's some unlucky X
 function still queued which tries to manipulate the window after it has been
 closed.
 
-[2003-03-03, 00.13-01.54] Added the B<--print> option, which can be extremely
+[2003-03-03, 00:13-01:54] Added the B<--print> option, which can be extremely
 useful in scripts and other programs where you want to use a menu for some kind
 of input, rather than for a platform from which to fire off some program. Added
 L<"BUGS"> section below. Added the B<--debug> option which (as of now) dumps
 some info on the events (key/mouse clicks focusing etc.) intercepted from X.
 
-[2003-03-08, 11.21-11.29] Added a declaration of the subroutine `strcasecmp'
+[2003-03-08, 11:21-11:29] Added a declaration of the subroutine `strcasecmp'
 and thus eliminating a compiler warning. Program worked flawlessy even before
 (and still does) but with less annoyance for me.. =|:-) The declaration thingy
 was found on the 'net by googling for "strcasetmp introduction". Made arguments
 to B<--align> and B<--style> case insensetive.
 
-[2003-03-08, 20.27-22.58] Found and killed a bug which made X resources
+[2003-03-08, 20:27-22:58] Found and killed a bug which made X resources
 override the command line options.
 
-[2003-03-09, 07.04-10.15] Totally fixed the bug mentioned in previous comment.
-Added short options B<-l>, B<c> and B<-r> (for left, center and right aligning
+[2003-03-09, 07:04-10:15] Totally fixed the bug mentioned in previous comment.
+Added short options B<-l>, B<-c> and B<-r> (for left, center and right aligning
 of the menu text), and changed B<--label> to B<--title>. Also changed the size
 of the menu window so that it will always be an even number of text lines
 (looks pretty ogly when only half or a third of the last item can be seen).
@@ -1611,54 +1630,65 @@ Opening of window could be still more optimal though (the problem still arises
 when X refuses to open the window with the requested height, e.g. when one uses
 frames in ratpoison).
 
-[2003-03-10, 00.53-04.07] v2.0 - Optimized menu redrawing routine for the
+[2003-03-10, 00:53-04:07] v2.0 - Optimized menu redrawing routine for the
 non-scrolling case (used to flicker quite nastily when moving down large menus
 fast). Also finally got around to add scrolling capacity for menus too large to
 fit all at once. Added the related B<--scroll-offset> option and X resource as
 well.
 
-[2003-03-10, 22.10-23.00] v2.0.1 - Begun fixxing window size bug (interrupted
+[2003-03-10, 22:10-23:00] v2.0.1 - Begun fixxing window size bug (interrupted
 by Buffy - The Vampire Slayer).
 
-[2003-03-11, 00.31-04.25]
+[2003-03-11, 00:31-04:25]
 
-[2003-03-12, 00.13-01.14] v2.0.2 - Fixed weird update/redraw bug thingy. Used
+[2003-03-12, 00:13-01:14] v2.0.2 - Fixed weird update/redraw bug thingy. Used
 to get a totally black empty menu sometimes. Bug probably introduced in last
 session, and now deceased.
 
-[2003-03-19, 10.48-12.38]
+[2003-03-19, 10:48-12:38]
 
-[2003-03-19, 14.17-20.18] v2.1 - Wrote better handling of X resources, comlete
+[2003-03-19, 14:17-20:18] v2.1 - Wrote better handling of X resources, comlete
 with readable B<--debug> output and all.
 
-[2003-03-19, 22.40-09.05] v2.2 - Implemented mouse support. Made B<--item=0>
+[2003-03-19, 22:40-09:05] v2.2 - Implemented mouse support. Made B<--item=0>
 special. Bugfix: Now opens a window of the correct size by telling X that it
 wants a window whose vertical size should be any number of pixels evenly
 dividable by the font height. (B<Rxvt> served as a source of inspiration for
 this feature.) Also added keys Home/PgUp for going to the first item in the
 menu and End/PgDn to go to the last. B<--class> now really works.
 
-[2003-03-24, 15.41-16.32] v2.2.1 - Added options B<--mouse> and B<--no-mouse>
+[2003-03-24, 15:41-16:32] v2.2.1 - Added options B<--mouse> and B<--no-mouse>
 and the corresponding X resource B<ratmen.mouse> -- mouse is enabled by
 default. Removed the short options B<-u> and B<-U> (synonyms for
 B<--unfocus-exit> and B<--no-unfocus-exit>) as to not encourage the
 useage of those options on the command line (should be set using X resources).
 
-[2003-03-28, 11.29-11.32] v2.2.2 - Killed bug which didn't allow spaces to be
+[2003-03-28, 11:29-11:32] v2.2.2 - Killed bug which didn't allow spaces to be
 used in font names (this little bugfix also makes it quite necessary not to end
 your X resources in any extraneous spaces).
 
+[2007-10-25, 11:23-14:13] v2.2.3 -- Bugfix. Changed a couple of latin-1
+characters 173 (soft hyphens) to the minus signs the should have been all the
+time. Added the "FUTURE" heading above. Rewrote the Makefile.
+
 
 =head1 BUGS
 
 =over 4
 
+=item o
+
+When no item is selected (i.e. on startup with -i0) and you hover the menu with
+the pointer, and then remove the pointer, the first item becomes selected.
+
+
 =item o
 
 The B<snazzy> mode. I'm probably going to remove it sometime in the future.
 B<I> don't use it and I don't see any reason why anybody else should want to
 either... (Especially not now that we got scrollable B<dreary> mode..)
 
+
 =item o
 
 Generates a `BadWindow' when dying of unfocus under the B<--unfocus-exit>
@@ -1688,13 +1718,63 @@ to select the first item on the menu. (I'll probably fix this by removing the
 
 These are the things a want to do next in approximate order of priority:
 
-  o configurable keys for up/down choose etc.
-  o manually defined key shortcuts
-  o automatically generated key shortcuts
-  o multiple selection mode (space=select; enter=finalize)
-    (introduce colors for marked entry and marked&selected entry)
-  o incremental search in menu (using bottom item space for input)
-  o replacing top/bottom item with arrow if scrollable in that direction
+
+=over 4
+
+=item o
+
+Parsing of standard input (so one may use B<ratmen> as a magic number) for
+interpreting menu files. If this is to work, maybe some special command line
+argument parsing is needed? (I strongly suspect that magic number arguments are
+not passed in the same fashion, as if given on the command line..)
+
+
+=item o
+
+Option for outputting number of selected menu item (B<--print-number>/B<-P>
+maybe?).
+
+
+=item o
+
+Change B<--print> behaviour to include B<-d>: or something automatically (so
+that menu item text is default output).
+
+
+=item o
+
+User-configurable keys for up/down choose etc.
+
+=item o
+
+Menu shortcuts (defined in the menu) -- if autogenerated shortcuts are good
+enough, this might not be needed.
+
+
+=item o
+
+Automatically generated key shortcuts (E.g. numbers 0-9 for items 1-10, or
+letter 'a' letter to loop through all items begginning with an 'a', or "/" and
+"?" to do incremental forward/backward search...)
+
+
+=item o
+
+Multiple selection mode (space=select; enter=finalize) (introduce colors for
+marked entry and marked & selected entry).
+
+
+=item o
+
+Incremental search in menu (using bottom item space for input).
+
+
+=item o
+
+Replacing top/bottom item with arrow (or something) if scrollable in that
+direction.
+
+=back
 
 =cut
 
